#!/usr/bin/perl -w
# dateTill:  date "+%Y-%m-%d-%H:%M:%S"

use strict;
use Date::Manip qw(ParseDate UnixDate DateCalc Date_SetTime);
use Getopt::Long;

my ($dateFrom,$dateTill);

if(!GetOptions('dtill=s' => \$dateTill
           )) {
	print 'Usage: ./queuereport.pl --dtill dateTill';
	exit(0); 
}

$dateTill = ParseDate($dateTill);

# You need to redefine the following custom variables:

my $dirName  = "/var/CommuniGate/Accounts/pbx.macnt/account.web/private/queuelogs/"; # the queuelogs directory path
my $subFileName = '/var/CommuniGate/Submitted/report'.int(rand(1000));               # the temp file name inside the CGPro Submitted directory
my $sendFrom = 'pbx@domain.name';                                                    # an address from which the report is sent
my $sendTo   = 'support@domain.name';                                                # the report recipient addresses.

# End of custom variables

my %totalCallNum;
my %successCallNum;
my %failedCallNum;
my %failedCallList;
my $queueName;

if($dateTill) {
	$dateFrom = DateCalc($dateTill,"- 24 hour");
} else {
	print qq/ERROR: incorrect date format\n/;
	exit(0);
}

my $currentDate = Date_SetTime($dateFrom,"00:00:00");

while($currentDate le $dateTill) {
	processDate($currentDate);
	$currentDate = DateCalc($currentDate,"+ 1 day");
}

unless( open(WFILE,">$subFileName.tmp")) {
	print qq/ REJECTED can't create a file: $!\n/;
  die "can't create a file: $!";
}

format WFILE =
 @<<<<<<<<<<<<<<<<<<<< @>>>>>>>>>>>> @>>>>>      @>>>>>  @>>>>>>>>
$$_{"timeStamp"},$$_{"fromNumber"},$$_{"position"},$$_{"total"},$$_{"qTime"}
.

print WFILE qq/Return-Path: <>\n/;
print WFILE qq/Envelope-To: $sendTo\n/;
print WFILE qq/From: <$sendFrom>\n/;
print WFILE qq/To: $sendTo\n/;
print WFILE "Content-Type: text/plain;charset=utf-8; format=\"flowed\"\n";
print WFILE qq/Content-Transfer-Encoding: 8bit\n/;
print WFILE "Subject: Queue Report ".UnixDate($dateFrom,'%Y/%m/%d %H:%M:%S')." - ".UnixDate($dateTill,'%Y/%m/%d %H:%M:%S')."\n\n";
print WFILE "\n Отчет по работе операторов с ".UnixDate($dateFrom,'%Y-%m-%d %H:%M:%S')." до ".UnixDate($dateTill,'%Y-%m-%d %H:%M:%S')."\n";
print WFILE "_______________________________________________________________________\n";

foreach $queueName (keys %totalCallNum) {	
	print WFILE qq/\nНазвание группы:         $queueName\n\n/;
	print WFILE qq/Количество звонков:       $totalCallNum{$queueName}\n/;
	print WFILE qq/Отвеченные:               $successCallNum{$queueName}\n/;
	print WFILE qq/Пропущенные:              $failedCallNum{$queueName}\n/;
	print WFILE qq/Детализация пропущенных звонков:\n/;
	print WFILE qq/_______________________________________________________________________\n/;
	print WFILE qq/ Дата       Время         Телефонный    Позиция     Длина    Время\n/;
	print WFILE qq/ звонка     звонка           номер      в очереди   очереди  ожидания\n/;
	print WFILE qq/_______________________________________________________________________\n/;
	foreach (@{$failedCallList{$queueName}}) {write WFILE;}
	print WFILE qq/_______________________________________________________________________\n/;
}

close(WFILE);
unless(rename("$subFileName.tmp","$subFileName.sub")) {
  unlink("$subFileName.tmp");
}


#######################################
# Subs
#######################################

sub processDate {

	my $currDate = shift(@_);
	my $fileName = $dirName.UnixDate($currDate,'%Y-%m/%d.txt');
	
	unless(open(FILE,$fileName)) {
  	return undef;
	}

	while(<FILE>) {
  	chomp;
		my ($timeStamp,$queueName,$queueLength,$from,$callID,$t1,$t2,$t3,$status,$position,$total,$agentIP) = split(/\t/);
		$timeStamp = Date_SetTime($currDate,$timeStamp);
		if($timeStamp ge $dateFrom and $timeStamp lt $dateTill) {
			
			$totalCallNum{$queueName}   = 0 unless defined $totalCallNum{$queueName};
			$successCallNum{$queueName} = 0 unless defined $successCallNum{$queueName};
			$failedCallNum{$queueName}  = 0 unless defined $failedCallNum{$queueName};
			
			if($status eq 'OK') {
				$agentIP = $position;
				$totalCallNum{$queueName}   ++;
				$successCallNum{$queueName} ++;
			} elsif($status eq 'left' or $status eq 'fail') {
				$totalCallNum{$queueName}   ++;
				$failedCallNum{$queueName}  ++;
				push(@{$failedCallList{$queueName}}, {timeStamp => UnixDate($timeStamp,'%Y-%m-%d %H:%M:%S'), fromNumber => $from, position => $position, total => $total, qTime => $t1});				
			} else {
				print "Incorrect status\n";
			}
		}
	}
	close(FILE);
}